/*
  Copyright (c) 2014, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once


#include "Direction.hpp"
#include "Position.hpp"


#include <string>
#include <utility>


namespace RPGEdit {

namespace Model {

class Entity {
private:
	const std::string name;

	Position pos;
	Direction direction;

	uint64_t transitionStart = 0;
	uint64_t transitionEnd = 0;

	Position translate(Direction dir, float amount) const {
		Position p = pos;

		switch (dir) {
		case NORTH:
			p.y -= amount;
			break;

		case EAST:
			p.x += amount;
			break;

		case SOUTH:
			p.y += amount;
			break;

		case WEST:
			p.x -= amount;
		}

		return p;
	}

public:
	Entity(const std::string &name0)
		: name(name0), direction(NORTH) {
	}

	const std::string & getName() const {
		return name;
	}

	Position getPosition(uint64_t time) const {
		if (hasTransition())
			if (time <= transitionStart)
				return pos;
			else if (time >= transitionEnd)
				return translate(direction, 1);
			else
				return translate(direction, float(time-transitionStart)/float(transitionEnd-transitionStart));
		else
			return pos;
	}

	Direction getDirection() const {
		return direction;
	}

	void moveTo(Position newPos) {
		pos = newPos;
		transitionStart = transitionEnd = 0;
	}

	void move(Direction dir, uint64_t start, uint64_t end) {
		if (hasTransition())
			return;

		direction = dir;

		transitionStart = start;
		transitionEnd = end;
	}

	void finishTransition() {
		if (hasTransition())
			moveTo(translate(direction, 1));
	}

	bool hasTransition() const {
		return transitionEnd;
	}
};

}

}
